package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.ruoyi.app.annotation.LoginAppUser;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.domain.TbFamilyMember;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbFamilyMemberService;
import com.ruoyi.app.service.ITbFamilyService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.bean.BeanUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.aspectj.weaver.bcel.AtAjAttributes;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 家族成员管理
 *
 * @author Alex
 * @date 2020-09-24
 */
@Api(value = "家族成员管理",tags = "家族成员管理")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/member" )
public class FamilyMemberController extends AppBaseController {

    private final ITbFamilyMemberService memberService;
    private final ITbAppUserService appUserService;
    private final ITbFamilyService familyService;
    private final ITbMemberMiddleService middleService;

    @ApiOperation("我的家族成员列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "当前页",paramType="int"),
            @ApiImplicitParam(name = "pageSize", value = "每页行数",paramType="int")
    })
    @GetMapping("/list")
    public TableDataInfo list(Integer pageNum, Integer pageSize) {
        // 登录用户
        TbAppUser user = getLoginUser().getUser();
        // 获取成员信息
        AppMemberVo memberVo = appUserService.getAppMember(user.getId().toString(),"");
        if (memberVo == null) {
            return new TableDataInfo();
        }
        // 获取默认家族信息
        TbFamily family = familyService.myFamily(memberVo.getMemberId());
        // 获取家族关联成员的信息
        List<TbMemberMiddle> middles = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
            .eq(TbMemberMiddle::getFamilyId, family.getId())
        );
        if (middles.size() <= 0) {
            return new TableDataInfo();
        }
        List<Long> ids = new ArrayList<>();
        middles.forEach(item -> {
            ids.add(item.getMemberId());
        });
        // 根据家族获取成员信息
        pageNum = pageNum == null ? 1 : pageNum;
        pageSize = pageSize == null ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize, "py_first asc");
        List<TbFamilyMember> list = memberService.list(new LambdaQueryWrapper<TbFamilyMember>()
            .in(TbFamilyMember::getId, ids)
        );
        return getDataTable(list);
    }

    @ApiOperation("家族树")
    @GetMapping("/tree")
    public AjaxResult tree(Long familyId){
        if (familyId == null) {
            return AjaxResult.error("家族id不能为空");
        }
        return AjaxResult.success(memberService.getFamilyTree(familyId));
    }

    @ApiOperation("添加成员-家族树")
    @PostMapping("/addToTree")
    public AjaxResult addToTree(@LoginAppUser TbAppUser user, @RequestBody TbMemberMiddle member){
        if (member == null) {
            return AjaxResult.error("数据为空");
        }
        if (member.getFamilyId()==null) {
            return AjaxResult.error("家族id不能为空");
        }
        if (member.getParentId()==null) {
            return AjaxResult.error("父辈不能为空");
        }
        if (member.getMemberId()==null) {
            return AjaxResult.error("姓名不能为空");
        }
        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,member.getFamilyId())
                .eq(TbMemberMiddle::getMemberId,member.getMemberId())
                .last("limit 1")
        );
        middle.setParentId(member.getParentId());
        if(!middleService.updateById(middle)){
            return AjaxResult.error("添加失败");
        }

        return AjaxResult.success("添加成功");
    }

    //TODO 逻辑删除家族树的成员（去掉parentId）；如果这一代只有他一个，则不允许删除，否则家族树会断开

}
